      subroutine richardson(psurf,ust,ttlev,qvlev,ulev,vlev,nuvz,
     +akz,bkz,hf,tt2,td2,h,wst,hmixplus,zbotcv,ztopcv,lconvection)
C                             i    i    i     i    i    i    i
C      i   i  i   i   i  o  o     o       o      o        i
*****************************************************************************
*                                                                           *
*     Calculation of mixing height based on the critical Richardson number. *
*     Calculation of convective time scale.                                 *
*     For unstable conditions, one iteration is performed. An excess        *
*     temperature (dependent on hf and wst) is calculated, added to the     *
*     temperature at the lowest model level. Then the procedure is repeated.*
*                                                                           *
*     Author: A. Stohl                                                      *
*                                                                           *
*     22 August 1996                                                        *
*                                                                           *
*     Literature:                                                           *
*     Vogelezang DHP and Holtslag AAM (1996): Evaluation and model impacts  *
*     of alternative boundary-layer height formulations. Boundary-Layer     *
*     Meteor. 81, 245-269.                                                  *
*                                                                           *
*     Update: 1999-02-01 by G. Wotawa                                       *
*                                                                           *
*     Two meter level (temperature, humidity) is taken as reference level   *
*     instead of first model level.                                         *
*     New input variables tt2, td2 introduced.                              *
* ------------------------------------------------------------------        *
*     Petra Seibert, Feb 2000:                                              *
*     New code added at the end for convection scheme                       *
*     new variables,zbotcv,ztoptv in parameter list                         *
*     they give bottom and top heights of the lowest unstable layer         *
*     if no such layer, they are set to -9999.                              *
*                                                                           *
*****************************************************************************
*                                                                           *
* Variables:                                                                *
* h                          mixing height [m]                              *
* hf                         sensible heat flux                             *
* psurf                      surface pressure at point (xt,yt) [Pa]         *
* tv                         virtual temperature                            *
* wst                        convective velocity scale                      *
* zbotcv                     bottom of convective mixing layer              *
* ztopcv                     top of convective mixing layer                 *
*                                                                           *
* Constants:                                                                *
* ric                        critical Richardson number                     *
*                                                                           *
*****************************************************************************

      include 'includepar'

      integer i,k,nuvz,itmax,iter,kmix,lconvection
      real tv,tvold,zref,z,zold,pint,pold,theta,thetaref,ri
      real akz(nuvz),bkz(nuvz),ulev(nuvz),vlev(nuvz),hf,wst,tt2,td2,ew
      real psurf,ust,ttlev(nuvz),qvlev(nuvz),h,const,ric,b,excess,bs
      real thetaold,zl,ul,vl,thetal,ril,hmixplus,wspeed,bvfsq,bvf,rhsat
      real zbotcv,ztopcv,f_qvsat,plev,thvgr
      real t,p,rh,rhold,rmx,q,z1,p1,tv1,thv1,thev1,rh1,ql,thevl,rhl
      real qmin,rhmin,thev2,thv2,rh2,p2,z2,rxl,pl,dz,thevgr,rx,tv2
      real theta1,theta2,zl1,zl2,thetam,th,rmix,the,tvirt,tnvirt
      parameter(const=r_air/ga,ric=0.25,b=100.,bs=8.5,itmax=3)
      parameter(rhsat=.8)
      logical start


*************************************************************************
C Statement functions (Petra Seibert)

      plev( k ) = (akz(k) + bkz(k)*psurf)
      th( t, p ) = t * ( 100000. / p )**0.286
      rmix( k ) = qvlev(k) / ( 1. - qvlev(k) )
      the( theta, rmx, t) = theta * exp( 2490. * rmx / t )
      tvirt( t, q ) = t * (1. + 0.608*q )
      tnvirt (t, q ) = t / (1. + 0.608*q )
**************************************************************************
 
      excess=0.0
      iter=0
 
C Compute virtual temperature and virtual potential temperature at
C reference level (2 m)
******************************************************************
 
30    iter=iter+1
 
      pold=akz(1)
      tvold=tt2*(1.+0.378*ew(td2)/psurf)
      zold=2.0
      zref=zold
      rhold=ew(td2)/ew(tt2)
 
      thetaref=tvold*(100000./pold)**(r_air/cpa)+excess
      thetaold=thetaref
 
 
C Integrate z up to one level above zt
**************************************
 
      do 10 k=2,nuvz-1
        pint=akz(k)+bkz(k)*psurf  ! pressure on model layers
        tv=ttlev(k)*(1.+0.608*qvlev(k))
 
        if (abs(tv-tvold).gt.0.2) then
          z=zold+const*log(pold/pint)*(tv-tvold)/log(tv/tvold)
        else
          z=zold+const*log(pold/pint)*tv
        endif
 
        theta=tv*(100000./pint)**(r_air/cpa)
! Petra
        rh = qvlev(k) / f_qvsat( pint, ttlev(k) )
 
 
Calculate Richardson number at each level
*****************************************
 
        ri=ga/thetaref*(theta-thetaref)*(z-zref)/
     +  max(((ulev(k)-ulev(2))**2+(vlev(k)-vlev(2))**2+b*ust**2),0.1)
 
C  addition of second condition: MH should not be placed in an
C  unstable layer (PS / Feb 2000)
        if (ri.gt.ric .and. thetaold.lt.theta) goto 20
 
        tvold=tv
        pold=pint
        rhold=rh
        thetaold=theta
10      zold=z
      
20    continue
 
C Determine Richardson number between the critical levels
*********************************************************

      zl1=zold
      theta1=thetaold
      do 15 i=1,20
        zl=zold+float(i)/20.*(z-zold)
        ul=ulev(k-1)+float(i)/20.*(ulev(k)-ulev(k-1))
        vl=vlev(k-1)+float(i)/20.*(vlev(k)-vlev(k-1))
        thetal=thetaold+float(i)/20.*(theta-thetaold)
        rhl=rhold+float(i)/20.*(rh-rhold)
        ril=ga/thetaref*(thetal-thetaref)*(zl-zref)/
     +  max(((ul-ulev(2))**2+(vl-vlev(2))**2+b*ust**2),0.1)
        zl2=zl
        theta2=thetal
        if (ril.gt.ric) goto 25
        zl1=zl
        theta1=thetal
15      continue
 
25    continue
      h=zl
      thetam=0.5*(theta1+theta2)
      wspeed=sqrt(ul**2+vl**2)                    ! Wind speed at z=hmix
      bvfsq=(ga/thetam)*(theta2-theta1)/(zl2-zl1) ! Brunt-Vaisala frequency
                                                  ! at z=hmix

C Under stable conditions, limit the maximum effect of the subgrid-scale topography
C by the maximum lifting possible from the available kinetic energy
***********************************************************************************

      if(bvfsq.le.0.) then
        hmixplus=9999.
      else
        bvf=sqrt(bvfsq)
        hmixplus=wspeed/bvf*convke                ! keconv = kinetic energy
      endif                                       ! used for lifting

 
C Calculate convective velocity scale
*************************************
 
      if (hf.lt.0.) then
        wst=(-h*ga/thetaref*hf/cpa)**0.333
        excess=-bs*hf/cpa/wst
        if (iter.lt.itmax) goto 30
      else
        wst=0.
      endif


C If convection parameterization is turned off, return
******************************************************

      if (lconvection.eq.2) return


C  convection scheme parameter calculation / Petra Seibert / 2-2000
*********************************************************************

C kmix is the layer just above hmix

      start = .true.
      zbotcv = -9999.
      ztopcv = -9999.
C      iko=0 ! from tests, remove later
      z1 = zold
      kmix = k
      p1 = plev(kmix)
      tv1 = tvirt( ttlev(kmix), qvlev(kmix) )
      thv1 = th( tv1, p1 )
      thev1 = the( thv1,  rmix(kmix), ttlev(kmix) )
      rh1 = qvlev(kmix) / f_qvsat( p1, ttlev(kmix) )

C Check for instability between h and kmix

      if (h .lt. z) then

C Theta_e at mixing height

        ql = qvlev(k-1)+float(i)/20.*(qvlev(k)-qvlev(k-1))
        pl = (pold + float(i)/20. * (p1-pold))*0.01
        rxl = ql / (1. - ql)
        thevl = the( thetal , rxl, tnvirt(thetal/th( 1., pl ), rxl) )
C         note that thetal is already a virtual temperature
        rhmin = min (rh1,rhl)

C Theta and theta_e gradients

        dz = z - h
        thvgr = (thv1 - thetal) / dz
        thevgr = (thev1 - thevl) / dz
        if (thvgr .lt. 0. .or. thevgr .lt. 0. and. rh .gt. rhmin) then
          zbotcv = h
          ztopcv = z
          start = .false.
        endif
      endif

C now continue above kmix

      do 40 k = kmix+1, nuvz
        p2 = plev(k)
        rx = rmix( k)
        tv2 = tvirt( ttlev(k), qvlev(k) )
        if ( abs(tv2 -tv1) .gt .0.01) then
          z2 = z1 + const * log(p1/p2) * (tv2-tv1) / log(tv2/tv1)
        else
          z2 = z1 + const * log(p1/p2) * tv2
        endif
        thv2 = th( tv2, p2 )
        thev2 = the( thv2, rx, ttlev(k) )
        rh2 = qvlev(k) / f_qvsat( p2, ttlev(k) )
        rhmin = min(rh1,rh2)
        qmin = min(qvlev(k),qvlev(k-1))
        dz = z2 - z1

C       Calculate gradients and update conv. zone limits

        thvgr =  (thv2 - thv1) / dz
        thevgr = (thev2 - thev1) / dz

c       look only at nearly-saturated regions for THev; discard
c       regions with very low spec. humidity as RH may be unreliable

        if (qmin .ge. 0.01e-3 .and. thvgr .lt. 0. .or.
     &    thevgr .lt. 0. .and. rhmin .gt. rhsat) then
          if (start) then
            zbotcv = z1
            ztopcv = z2
c             kconv1 = k-1
c             kconv2 = k
c             iko=iko+1
             start = .false.
           else
             ztopcv = z2
c             kconv2 = k
             goto 100

C        jump out of loop as we consider here only lowest unstable layer

           endif

         elseif (.not. start) then

           start = .true.

        endif

        p1 = p2
        thv1 = thv2
        thev1 = thev2
        tv1 = tv2
        z1 = z2
40    continue

100   continue
      return
      end
