      subroutine timemanager()                                  

********************************************************************************
*                                                                              *
* Handles the computation of trajectories, i.e. determines which               *
* trajectories have to be computed at what time.                               *
* Manages dry+wet deposition routines, radioactive decay and the computation   *
* of concentrations.                                                           *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     20 May 1996                                                              *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* DEP                .true. if either wet or dry deposition is switched on     *
* decay(maxspec) [1/s] decay constant for radioactive decay                    *
* DRYDEP             .true. if dry deposition is switched on                   *
* ideltas [s]        modelling period                                          *
* itime [s]          actual temporal position of calculation                   *
* ldeltat [s]        time since computation of radioact. decay of depositions  *
* loutaver [s]       averaging period for concentration calculations           *
* loutend [s]        end of averaging for concentration calculations           *
* loutnext [s]       next time at which output fields shall be centered        *
* loutsample [s]     sampling interval for averaging of concentrations         *
* loutstart [s]      start of averaging for concentration calculations         *
* loutstep [s]       time interval for which concentrations shall be calculated*
* lsampnext [s]      next time at which concentrations shall be sampled        *
* npoint(maxpart)    index, which starting point the trajectory has            *
*                    starting positions of trajectories                        *
* outnum             number of samples for each concentration calculation      *
* prob               probability of absorption at ground due to dry deposition *
* WETDEP             .true. if wet deposition is switched on                   *
* weight             weight for each concentration sample (1/2 or 1)           *
* uap(maxpart),ucp(maxpart),uzp(maxpart) = random velocities due to turbulence *
* us(maxpart),vs(maxpart),ws(maxpart) = random velocities due to interpolation *
* xtra1(maxpart), ytra1(maxpart), ztra1(maxpart) =                             *
*                    spatial positions of trajectories                         *
*                                                                              *
* Constants:                                                                   *
* maxpart            maximum number of trajectories                            *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer i,j,k,l,n,itime,nstop
      integer loutnext,loutstart,loutend,lsampnext
      integer ix,jy,kz,ldeltat,itage,nage
      real outnum,weight,prob(maxspec)
      real uap(maxpart),ucp(maxpart),uzp(maxpart),decfact
      real us(maxpart),vs(maxpart),ws(maxpart),cbt(maxpart)
      real drydeposit(maxspec),gridtotalunc,wetgridtotalunc
      real drygridtotalunc,xold,yold,zold
C     real xm,xm1
      logical initial
      save initial
      data initial/.false./



C First output for time 0, initialization of output grid
********************************************************

      loutnext=loutstep/2
      outnum=0.
      loutstart=loutnext-loutaver/2
      loutend=loutnext+loutaver/2
      do 31 k=1,nspec
        do 33 i=1,numreceptor
33        creceptor(i,k)=0.
        do 31 nage=1,nageclass
          do 31 jy=0,numygrid-1
            do 31 ix=0,numxgrid-1
              wetgrid(ix,jy,k,nage)=0.
              drygrid(ix,jy,k,nage)=0.
              do 31 kz=1,numzgrid
                fluxe(ix,jy,kz,k,nage)=0.
                fluxw(ix,jy,kz,k,nage)=0.
                fluxn(ix,jy,kz,k,nage)=0.
                fluxs(ix,jy,kz,k,nage)=0.
                fluxu(ix,jy,kz,k,nage)=0.
                fluxd(ix,jy,kz,k,nage)=0.
                do 31 l=1,nclassunc
31                gridunc(ix,jy,kz,k,l,nage)=0.


C Initialize random velocities
******************************

      do 15 j=1,numpart
        uap(j)=0.
        ucp(j)=0.
        uzp(j)=0.
        us(j)=0.
        vs(j)=0.
15      ws(j)=0.


***********************************************************************
C Loop over the whole modelling period in time steps of mintime seconds
***********************************************************************

      do 10 itime=0,ideltas,lsynctime

C Get necessary wind fields if not available
********************************************

        call getfields(itime,nstop)
        if (nstop.gt.1) stop 'NO METEO FIELDS AVAILABLE'



C Computation of wet deposition every loutsample seconds
********************************************************

        if (WETDEP.and.(ldirect*itime.ge.ldirect*loutsample).and.
     +  (mod(itime,loutsample).eq.0).and.initial) then
          call wetdepo(itime,loutsample,loutnext)
        endif


C If middle of averaging period of output fields is reached, accumulated
C deposited mass radioactively decays 
************************************************************************

        if (DEP.and.(itime.eq.loutnext)) then 
          do 41 k=1,nspec
            if (decay(k).gt.0.) then
              do 42 nage=1,nageclass
                do 42 jy=0,numygrid-1
                  do 42 ix=0,numxgrid-1
                    wetgrid(ix,jy,k,nage)=wetgrid(ix,jy,k,nage)*
     +              exp(-1.*float(loutstep)*decay(k))
42                  drygrid(ix,jy,k,nage)=drygrid(ix,jy,k,nage)*
     +              exp(-1.*float(loutstep)*decay(k))
            endif
41          continue
        endif

!!! CHANGE: These lines may be switched on to check the conservation
!!! of mass within FLEXPART

c       if (mod(itime,loutsample).eq.0) then 
c          xm=0.
c          xm1=0.
c          do 247 j=1,numpart
c47          xm1=xm1+xmass1(j,1)
c          xm=xm1
c          do 248 nage=1,nageclass
c            do 248 ix=0,numxgrid-1
c              do 248 jy=0,numygrid-1
c48              xm=xm+wetgrid(ix,jy,1,nage)+drygrid(ix,jy,1,nage)
c          write(*,'(i6,4f10.3)') itime,xm,xm1
c       endif
!!! CHANGE

          
C Check whether particle densities are to be calculated
*******************************************************

        if ((ldirect*itime.ge.ldirect*loutstart).and.
     +  (ldirect*itime.le.ldirect*loutend)) then ! add to grid
          if (mod(itime-loutstart,loutsample).eq.0) then
            lsampnext=itime+loutsample
            if ((itime.eq.loutstart).or.(itime.eq.loutend)) then
              weight=0.5
            else
              weight=1.0
            endif
            outnum=outnum+weight
            call gridconc(itime,weight)
          endif



C Output and reinitialization of grid
C If necessary, first sample of new grid is also taken
******************************************************

          if ((itime.eq.loutend).and.(outnum.gt.0.)) then
            call gridout(itime,outnum,gridtotalunc,wetgridtotalunc,
     +      drygridtotalunc)
            if (iflux.eq.1) call fluxout(itime)
            write(*,45) itime,numpart,gridtotalunc,wetgridtotalunc,
     +      drygridtotalunc
45          format(i8,' SECONDS SIMULATED: ',i8,
     +      ' PARTICLES:    Uncertainty: ',3f7.3)
            if (ipout.eq.1) call partout(itime)    ! dump particle positions
            loutnext=loutnext+loutstep
            loutstart=loutnext-loutaver/2
            loutend=loutnext+loutaver/2
            if (itime.eq.loutstart) then
              weight=0.5
              outnum=outnum+weight
              call gridconc(itime,weight)
            endif


C Check, whether particles are to be split
******************************************

            n=numpart
            do 30 j=1,numpart
              if (ldirect*itime.ge.ldirect*itrasplit(j)) then
                if (n.lt.maxpart) then
                  n=n+1
                  itrasplit(j)=2*(itrasplit(j)-itramem(j))+itramem(j)
                  itrasplit(n)=itrasplit(j)
                  kind(n)=kind(j)
                  itramem(n)=itramem(j)
                  itra1(n)=itra1(j)
                  idt(n)=idt(j)
                  npoint(n)=npoint(j)
                  nclass(n)=nclass(j)
                  xtra1(n)=xtra1(j)
                  ytra1(n)=ytra1(j)
                  ztra1(n)=ztra1(j)
                  uap(n)=uap(j)
                  ucp(n)=ucp(j)
                  uzp(n)=uzp(j)
                  us(n)=us(j)
                  vs(n)=vs(j)
                  ws(n)=ws(j)
                  cbt(n)=cbt(j)
                  do 35 k=1,nspec
                    xmass1(j,k)=xmass1(j,k)/2.
35                  xmass1(n,k)=xmass1(j,k)
                endif
              endif
30            continue
            numpart=n
          endif
        endif
        

        if (itime.eq.ideltas) goto 99

C Compute interval since radioactive decay of deposited mass was computed
*************************************************************************

        if (itime.lt.loutnext) then
          ldeltat=itime-(loutnext-loutstep)
        else                                  ! first half of next interval
          ldeltat=itime-loutnext
        endif


C Loop over all particles
*************************

        do 20 j=1,numpart

C Determine age class of the particle
          itage=abs(itra1(j)-itramem(j))
          do 36 nage=1,nageclass
            if (itage.lt.lage(nage)) goto 37
36          continue
37        continue

C Terminate particles that are older than the maximum allowed age
*****************************************************************

          if (itage.gt.lage(nageclass)) itra1(j)=-999999999

C If integration step is due, do it
***********************************

          if (itra1(j).eq.itime) then
            if (initial.eqv..false.) initial=.true.

C Initialize newly released particle
************************************

            if ((itramem(j).eq.itime).or.(itime.eq.0))
     +      call initialize(kind(j),itime,idt(j),uap(j),ucp(j),uzp(j),
     +      us(j),vs(j),ws(j),xtra1(j),ytra1(j),ztra1(j),cbt(j))

C Integrate Lagevin equation for lsynctime seconds
**************************************************

            xold=xtra1(j)
            yold=ytra1(j)
            zold=ztra1(j)
            call advance(kind(j),itime,idt(j),uap(j),ucp(j),uzp(j),
     +      us(j),vs(j),ws(j),nstop,xtra1(j),ytra1(j),ztra1(j),prob,
     +      cbt(j))


C Calculate the gross fluxes
****************************

            if (iflux.eq.1) call calcfluxes(nage,j,xold,yold,zold)


C Determine, when next time step is due
C If trajectory is terminated, mark it
***************************************

            if (nstop.gt.1) then
              itra1(j)=-999999999
            else
              itra1(j)=itime+lsynctime


C Dry deposition and radioactive decay for each species
*******************************************************

              do 23 k=1,nspec
                if (decay(k).gt.0.) then             ! radioactive decay
                  decfact=exp(-1.*float(lsynctime)*decay(k))
                else
                  decfact=1.
                endif

                if (DRYDEPSPEC(k)) then        ! dry deposition
                  drydeposit(k)=xmass1(j,k)*prob(k)*decfact*
     +            float(ldirect)
                  xmass1(j,k)=xmass1(j,k)*(1.-prob(k))*decfact
                  if (decay(k).gt.0.) then   ! correct for decay (see wetdepo)
                    drydeposit(k)=drydeposit(k)*
     +              exp(float(ldeltat)*decay(k))
                  endif
                else                           ! no dry deposition
                  xmass1(j,k)=xmass1(j,k)*decfact
                endif
23              continue

              if (DRYDEP) call drydepokernel(nclass(j),drydeposit,
     +        xtra1(j),ytra1(j),nage)
            endif

          endif

20        continue                         ! end of loop over particles
          

10      continue                           ! end of loop over simulation time

99    continue

      write(*,*) 'itime before partout=',itime
      if (ipout.eq.2) call partout(itime)     ! dump particle positions

      return
      end
