      subroutine verttransform(n,uuh,vvh,tth,qvh,wwh)
C                              i  i   i   i   i   i
********************************************************************************
*                                                                              *
*     This subroutine transforms temperature, dew point temperature and        *
*     wind components from eta to meter coordinates.                           *
*     The vertical wind component is transformed from Pa/s to m/s using        *
*     the conversion factor pinmconv.                                          *
*     In addition, this routine calculates vertical density gradients          *
*     needed for the parameterization of the turbulent velocities.             *
*                                                                              *
*     Author: A. Stohl, G. Wotawa                                              *
*                                                                              *
*     12 August 1996                                                           *
*     Update: 16 January 1998                                                  *
*                                                                              *
*     Major update: 17 February 1999                                           *
*     by G. Wotawa                                                             *
*                                                                              *
*     - Vertical levels for u, v and w are put together                        *
*     - Slope correction for vertical velocity: Modification of calculation    *
*       procedure                                                              *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* nx,ny,nz                        field dimensions in x,y and z direction      *
* uu(0:nxmax,0:nymax,nzmax,2)     wind components in x-direction [m/s]         *
* vv(0:nxmax,0:nymax,nzmax,2)     wind components in y-direction [m/s]         *
* ww(0:nxmax,0:nymax,nzmax,2)     wind components in z-direction [deltaeta/s]  *
* tt(0:nxmax,0:nymax,nzmax,2)     temperature [K]                              *
* ps(0:nxmax,0:nymax,2)           surface pressure [Pa]                        *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer ix,jy,kz,iz,n,kmin,kl,klp,ix1,jy1,ixp,jyp
      real uvzlev(nuvzmax),rhoh(nuvzmax),pinmconv(nzmax)
      real ew,pint,tv,tvold,pold,const,dz1,dz2,dz,pih,ui,vi
      real xlon,ylat,xlonr,dzdx,dzdy
      real dzdx1,dzdx2,dzdy1,dzdy2
      real uuaux,vvaux,uupolaux,vvpolaux,ddpol,ffpol,wdummy
      real uuh(0:nxmax-1,0:nymax-1,nuvzmax)
      real vvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real tth(0:nxmax-1,0:nymax-1,nuvzmax)
      real qvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real wwh(0:nxmax-1,0:nymax-1,nwzmax)
      real wzlev(nwzmax),uvwzlev(0:nxmax-1,0:nymax-1,nzmax)
      parameter(const=r_air/ga)
      parameter(pih=pi/180.)

      logical init
      save init
      data init/.true./


**************************************************************************
* If verttransform is called the first time, initialize heights of the   *
* z levels in meter. The heights are the heights of model levels, where  *
* u,v,T and qv are given, and of the interfaces, where w is given. So,   *
* the vertical resolution in the z system is doubled. As reference point,*
* the lower left corner of the grid is used.                             *
* Unlike in the eta system, no difference between heights for u,v and    *
* heights for w exists.                                                  *
**************************************************************************

      if (init) then
        tvold=tt2(0,0,1,n)
        pold=akz(1)
        height(1)=0.

        do 5 kz=2,nz
          pint=akz(kz)+bkz(kz)*ps(0,0,1,n)
          tv=tth(0,0,kz)*(1.+0.608*qvh(0,0,kz))

          if (abs(tv-tvold).gt.0.2) then
            height(kz)=
     +      height(kz-1)+const*log(pold/pint)*
     +      (tv-tvold)/log(tv/tvold)
          else
            height(kz)=height(kz-1)+
     +      const*log(pold/pint)*tv
          endif

          tvold=tv
5         pold=pint

C Determine highest levels that can be within PBL
*************************************************

        do 8 kz=1,nz
          if (height(kz).gt.hmixmax) then
            nmixz=kz
            goto 9
          endif
8         continue
9       continue

        init=.false.

      endif


C Loop over the whole grid
**************************

      do 10 jy=0,ny-1
        do 10 ix=0,nx-1

          tvold=tt2(ix,jy,1,n)*(1.+0.378*ew(td2(ix,jy,1,n))/
     &                                   ps(ix,jy,1,n))
          pold=akz(1)
          uvzlev(1)=0.
          wzlev(1)=0.
          uvwzlev(ix,jy,1)=0
          rhoh(1)=pold/(r_air*tvold)


C Compute heights of eta levels
*******************************

          do 20 kz=2,nuvz
            pint=akz(kz)+bkz(kz)*ps(ix,jy,1,n)
            tv=tth(ix,jy,kz)*(1.+0.608*qvh(ix,jy,kz))
            rhoh(kz)=pint/(r_air*tv)

            if (abs(tv-tvold).gt.0.2) then
              uvzlev(kz)=uvzlev(kz-1)+const*log(pold/pint)*
     +        (tv-tvold)/log(tv/tvold)
            else
              uvzlev(kz)=uvzlev(kz-1)+const*log(pold/pint)*tv
            endif
            wzlev(kz)=uvzlev(kz)
            uvwzlev(ix,jy,kz)=uvzlev(kz)
           
            tvold=tv
20          pold=pint

C pinmconv=(h2-h1)/(p2-p1)

          pinmconv(1)=(uvwzlev(ix,jy,2)-uvwzlev(ix,jy,1))/
     +    ((aknew(2)+bknew(2)*ps(ix,jy,1,n))-
     +    (aknew(1)+bknew(1)*ps(ix,jy,1,n)))
          do 24 kz=2,nz-1
            pinmconv(kz)=(uvwzlev(ix,jy,kz+1)-uvwzlev(ix,jy,kz-1))/
     +      ((aknew(kz+1)+bknew(kz+1)*ps(ix,jy,1,n))-
     +      (aknew(kz-1)+bknew(kz-1)*ps(ix,jy,1,n)))
24        continue
          pinmconv(nz)=(uvwzlev(ix,jy,nz)-uvwzlev(ix,jy,nz-1))/
     +    ((aknew(nz)+bknew(nz)*ps(ix,jy,1,n))-
     +    (aknew(nz-1)+bknew(nz-1)*ps(ix,jy,1,n)))

C Levels, where u,v,t and q are given
*************************************

          uu(ix,jy,1,n)=uuh(ix,jy,1)
          vv(ix,jy,1,n)=vvh(ix,jy,1)
          tt(ix,jy,1,n)=tth(ix,jy,1)
          qv(ix,jy,1,n)=qvh(ix,jy,1)
          rho(ix,jy,1,n)=rhoh(1)
          uu(ix,jy,nz,n)=uuh(ix,jy,nuvz)
          vv(ix,jy,nz,n)=vvh(ix,jy,nuvz)
          tt(ix,jy,nz,n)=tth(ix,jy,nuvz)
          qv(ix,jy,nz,n)=qvh(ix,jy,nuvz)
          rho(ix,jy,nz,n)=rhoh(nuvz)
          kmin=2
          do 30 iz=2,nz-1
            do 35 kz=kmin,nuvz
              if(height(iz).gt.uvzlev(nuvz)) then
                uu(ix,jy,iz,n)=uu(ix,jy,nz,n)
                vv(ix,jy,iz,n)=vv(ix,jy,nz,n)
                tt(ix,jy,iz,n)=tt(ix,jy,nz,n)
                qv(ix,jy,iz,n)=qv(ix,jy,nz,n)
                rho(ix,jy,iz,n)=rho(ix,jy,nz,n)
                goto 30
              endif
              if ((height(iz).gt.uvzlev(kz-1)).and.
     +            (height(iz).le.uvzlev(kz))) then
               dz1=height(iz)-uvzlev(kz-1)
               dz2=uvzlev(kz)-height(iz)
               dz=dz1+dz2
               uu(ix,jy,iz,n)=(uuh(ix,jy,kz-1)*dz2+uuh(ix,jy,kz)*dz1)/dz
               vv(ix,jy,iz,n)=(vvh(ix,jy,kz-1)*dz2+vvh(ix,jy,kz)*dz1)/dz
               tt(ix,jy,iz,n)=(tth(ix,jy,kz-1)*dz2+tth(ix,jy,kz)*dz1)/dz
               qv(ix,jy,iz,n)=(qvh(ix,jy,kz-1)*dz2+qvh(ix,jy,kz)*dz1)/dz
               rho(ix,jy,iz,n)=(rhoh(kz-1)*dz2+rhoh(kz)*dz1)/dz
               kmin=kz
               goto 30
              endif
35            continue
30          continue


C Levels, where w is given
**************************

          ww(ix,jy,1,n)=wwh(ix,jy,1)*pinmconv(1)
          ww(ix,jy,nz,n)=wwh(ix,jy,nwz)*pinmconv(nz)
          kmin=2
          do 40 iz=2,nz
            do 45 kz=kmin,nwz
              if ((height(iz).gt.wzlev(kz-1)).and.
     +            (height(iz).le.wzlev(kz))) then
               dz1=height(iz)-wzlev(kz-1)
               dz2=wzlev(kz)-height(iz)
               dz=dz1+dz2
               ww(ix,jy,iz,n)=(wwh(ix,jy,kz-1)*dz2+wwh(ix,jy,kz)*dz1)/dz
     +         *pinmconv(iz)
               kmin=kz
               goto 40
              endif
45            continue
40          continue

C Compute density gradients at intermediate levels
**************************************************

          drhodz(ix,jy,1,n)=(rho(ix,jy,2,n)-rho(ix,jy,1,n))/
     +      (height(2)-height(1))
          do 50 kz=2,nz-1
50          drhodz(ix,jy,kz,n)=(rho(ix,jy,kz+1,n)-rho(ix,jy,kz-1,n))/
     +      (height(kz+1)-height(kz-1))
          drhodz(ix,jy,nz,n)=drhodz(ix,jy,nz-1,n)

10        continue


C If north pole is in the domain, calculate wind velocities in polar
C stereographic coordinates
********************************************************************
 
      if (nglobal) then
        do 74 jy=int(switchnorthg)-2,ny-1
          ylat=ylat0+float(jy)*dy
          do 74 ix=0,nx-1
            xlon=xlon0+float(ix)*dx
            do 74 iz=1,nz
74            call cc2gll(northpolemap,ylat,xlon,uu(ix,jy,iz,n),
     +        vv(ix,jy,iz,n),uupol(ix,jy,iz,n),
     +        vvpol(ix,jy,iz,n))


        do 76 iz=1,nz

* CALCULATE FFPOL, DDPOL FOR CENTRAL GRID POINT
          xlon=xlon0+float(nx/2-1)*dx
          xlonr=xlon*pi/180.
          ffpol=sqrt(uu(nx/2-1,ny-1,iz,n)**2+
     &               vv(nx/2-1,ny-1,iz,n)**2)
          if(vv(nx/2-1,ny-1,iz,n).lt.0.) then
            ddpol=atan(uu(nx/2-1,ny-1,iz,n)/
     &                 vv(nx/2-1,ny-1,iz,n))-xlonr
          else
            ddpol=pi+atan(uu(nx/2-1,ny-1,iz,n)/
     &                    vv(nx/2-1,ny-1,iz,n))-xlonr
          endif
          if(ddpol.lt.0.) ddpol=2.0*pi+ddpol
          if(ddpol.gt.2.0*pi) ddpol=ddpol-2.0*pi
 
* CALCULATE U,V FOR 180 DEG, TRANSFORM TO POLAR STEREOGRAPHIC GRID
          xlon=180.0
          xlonr=xlon*pi/180.
          ylat=90.0
          uuaux=-ffpol*sin(xlonr+ddpol)
          vvaux=-ffpol*cos(xlonr+ddpol)
          call cc2gll(northpolemap,ylat,xlon,uuaux,vvaux,uupolaux,
     +      vvpolaux)
      
          jy=ny-1
          do 76 ix=0,nx-1
            uupol(ix,jy,iz,n)=uupolaux
            vvpol(ix,jy,iz,n)=vvpolaux
76      continue
 
 
* Fix: Set W at pole to the zonally averaged W of the next equator-
* ward parallel of latitude
 
      do 85 iz=1,nz
          wdummy=0.
          jy=ny-2
          do 80 ix=0,nx-1
80          wdummy=wdummy+ww(ix,jy,iz,n)
          wdummy=wdummy/float(nx)
          jy=ny-1
          do 85 ix=0,nx-1
85          ww(ix,jy,iz,n)=wdummy
 
      endif 

 
C If south pole is in the domain, calculate wind velocities in polar
C stereographic coordinates
********************************************************************
 
      if (sglobal) then
        do 77 jy=0,int(switchsouthg)+3
          ylat=ylat0+float(jy)*dy
          do 77 ix=0,nx-1
            xlon=xlon0+float(ix)*dx
            do 77 iz=1,nz
77            call cc2gll(southpolemap,ylat,xlon,uu(ix,jy,iz,n),
     +        vv(ix,jy,iz,n),uupol(ix,jy,iz,n),
     +        vvpol(ix,jy,iz,n))
      
        do 79 iz=1,nz
 
* CALCULATE FFPOL, DDPOL FOR CENTRAL GRID POINT
          xlon=xlon0+float(nx/2-1)*dx
          xlonr=xlon*pi/180.
          ffpol=sqrt(uu(nx/2-1,0,iz,n)**2+
     &               vv(nx/2-1,0,iz,n)**2)
          if(vv(nx/2-1,0,iz,n).lt.0.) then
            ddpol=atan(uu(nx/2-1,0,iz,n)/
     &                 vv(nx/2-1,0,iz,n))+xlonr
          else
            ddpol=pi+atan(uu(nx/2-1,0,iz,n)/
     &                    vv(nx/2-1,0,iz,n))+xlonr
          endif
          if(ddpol.lt.0.) ddpol=2.0*pi+ddpol
          if(ddpol.gt.2.0*pi) ddpol=ddpol-2.0*pi
 
* CALCULATE U,V FOR 180 DEG, TRANSFORM TO POLAR STEREOGRAPHIC GRID
          xlon=180.0
          xlonr=xlon*pi/180.
          ylat=-90.0
          uuaux=+ffpol*sin(xlonr-ddpol)
          vvaux=-ffpol*cos(xlonr-ddpol)
          call cc2gll(northpolemap,ylat,xlon,uuaux,vvaux,uupolaux,
     +      vvpolaux)
      
          jy=0
          do 79 ix=0,nx-1
            uupol(ix,jy,iz,n)=uupolaux
79          vvpol(ix,jy,iz,n)=vvpolaux
 
 
* Fix: Set W at pole to the zonally averaged W of the next equator-
* ward parallel of latitude
 
        do 95 iz=1,nz
          wdummy=0.
          jy=1
          do 90 ix=0,nx-1
90          wdummy=wdummy+ww(ix,jy,iz,n)
          wdummy=wdummy/float(nx)
          jy=0
          do 95 ix=0,nx-1
95          ww(ix,jy,iz,n)=wdummy
      endif


      return
      end
